@@ -163,7 +163,7 @@ double novas_radio_refraction(double jd_tt, const on_surface *loc, enum novas_re
   static const double E[] = { 0.0, 46.625, 45.375, 4.1572, 1.4468, 0.25391, 2.2716, -1.3465, -4.3877, 3.1484, 4.520, -1.8982, 0.8900 };
 
   double E0, TK;
-  double y = 1.0, z;
+  double y, z;
   double poly;
   double fptem;
   double refraction;
@@ -176,6 +176,11 @@ double novas_radio_refraction(double jd_tt, const on_surface *loc, enum novas_re
     return NAN;
   }
 
+  if(loc->humidity < 0.0 || loc->humidity > 100.0) {
+    novas_set_errno(EINVAL, fn, "invalid humidity value: %g", loc->humidity);
+    return NAN;
+  }
+
   if(type == NOVAS_REFRACT_OBSERVED)
     return novas_inv_refract(novas_radio_refraction, jd_tt, loc, NOVAS_REFRACT_ASTROMETRIC, el);
 
