@@ -8,6 +8,14 @@
  *  can live in a separate, more manageably sized, module.
  */
 
+// We'll use gcc major version as a proxy for the glibc library to decide which feature macro to use.
+// gcc 5.1 was released 2015-04-22...
+#if defined(__GNUC__) && (__GNUC__ < 5)
+#  define _BSD_SOURCE             ///< strcasecmp() feature macro for glibc <= 2.19
+#else
+#  define _DEFAULT_SOURCE         ///< strcasecmp() feature macro starting glibc 2.20 (2014-09-08)
+#endif
+
 #include <math.h>
 #include <errno.h>
 #include <string.h>
@@ -15,6 +23,12 @@
 /// \cond PRIVATE
 #define __NOVAS_INTERNAL_API__    ///< Use definitions meant for internal use by SuperNOVAS only
 #include "novas.h"
+
+#if __Lynx__ && __powerpc__
+// strcasecmp() is not defined on PowerPC / LynxOS 3.1
+int strcasecmp(const char *s1, const char *s2);
+#endif
+
 /// \endcond
 
 /**
@@ -394,6 +408,16 @@ int cirs_to_tod(double jd_tt, enum novas_accuracy accuracy, const double *in, do
  * Transforms a rectangular equatorial (x, y, z) vector from the True of Date (TOD) reference
  * system to the Celestial Intermediate Reference System (CIRS) at the given epoch to the .
  *
+ * NOTES:
+ * <ol>
+ * <li>The accuracy of the output CIRS coordinates depends on how the input TOD coordinates
+ * were obtained. If TOD was calculated via the old (pre IAU 2006) method, using the Lieske et
+ * al. 1997 nutation model, then the limited accuracy of that model will affect the resulting
+ * coordinates. This is the case for the SuperNOVAS functions novas_geom_posvel() and
+ * novas_sky_pos() also, when called with `NOVAS_TOD` as the system, as well as all legacy NOVAS
+ * C functions that produce TOD coordinates.</li>
+ * </ol>
+ *
  * @param jd_tt     [day] Terrestrial Time (TT) based Julian date that defines
  *                  the output epoch. Typically it does not require much precision, and
  *                  Julian dates in other time measures will be unlikely to affect the
@@ -982,6 +1006,7 @@ int grav_undef(double jd_tdb, enum novas_accuracy accuracy, const double *pos_ap
  * @sa make_cat_entry()
  * @sa make_planet()
  * @sa make_ephem_object()
+ * @sa novas_geom_posvel()
  * @sa place()
  *
  * @since 1.1
@@ -1001,15 +1026,20 @@ int make_cat_object(const cat_entry *star, object *source) {
  *
  * @param name          Name of object. By default converted to upper-case, unless novas_case_sensitive()
  *                      was called with a non-zero argument. Max. SIZE_OF_OBJ_NAME long, including
- *                      termination.
- * @param num           Solar-system body ID number (e.g. NAIF)
+ *                      termination. If the ephemeris provider uses names, then the name should match
+ *                      those of the ephemeris provider -- otherwise it is not important.
+ * @param num           Solar-system body ID number (e.g. NAIF). The number should match the needs of the
+ *                      ephemeris provider used with NOVAS. (If the ephemeris provider is by name and not
+ *                      ID number, then the number here is not important).
  * @param[out] body     Pointer to structure to populate.
- * @return              0 if successful, or else -1 if the 'planet' pointer is NULL or the name
+ * @return              0 if successful, or else -1 if the 'body' pointer is NULL or the name
  *                      is too long.
  *
  *
+ * @sa set_ephem_provider()
  * @sa make_planet()
  * @sa make_cat_entry()
+ * @sa novas_geom_posvel()
  * @sa place()
  *
  * @since 1.0
@@ -1020,6 +1050,105 @@ int make_ephem_object(const char *name, long num, object *body) {
   return 0;
 }
 
+/**
+ * Sets a celestial object to be a Solar-system orbital body. Typically this would be used to define
+ * minor planets, asteroids, comets, or even planetary satellites.
+ *
+ * @param name          Name of object. It may be NULL if not relevant.
+ * @param num           Solar-system body ID number (e.g. NAIF). It is not required and can be set e.g. to
+ *                      -1 if not relevant to the caller.
+ * @param orbit         The orbital parameters to adopt. The data will be copied, not referenced.
+ * @param[out] body     Pointer to structure to populate.
+ * @return              0 if successful, or else -1 if the 'orbit' or 'body' pointer is NULL or the name
+ *                      is too long.
+ *
+ *
+ * @sa novas_orbit_posvel()
+ * @sa make_planet()
+ * @sa make_ephem_object()
+ * @sa novas_geom_posvel()
+ * @sa place()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+int make_orbital_object(const char *name, long num, const novas_orbital *orbit, object *body) {
+  static const char *fn = "make_orbital_object";
+  if(!orbit)
+    return novas_error(-1, EINVAL, fn, "Input orbital elements is NULL");
+  prop_error(fn, (make_object(NOVAS_ORBITAL_OBJECT, num, name, NULL, body) ? -1 : 0), 0);
+  body->orbit = *orbit;
+  return 0;
+}
+
+/**
+ * Populates a celestial object data structure with the parameters for a redhifted catalog
+ * source, such as a distant quasar or galaxy. It is similar to `make_cat_object()` except
+ * that it takes a Doppler-shift (z) instead of radial velocity and it assumes no parallax
+ * and no proper motion (appropriately for a distant redshifted source). The catalog name
+ * is set to `EXT` to indicate an extragalactic source, and the catalog number defaults to 0.
+ * The user may change these default field values as appropriate afterwards, if necessary.
+ *
+ * @param name        Object name (less than SIZE_OF_OBJ_NAME in length). It may be NULL.
+ * @param ra          [h] Right ascension of the object (hours).
+ * @param dec         [deg] Declination of the object (degrees).
+ * @param z           Redhift value (&lambda;<sub>obs</sub> / &lambda;<sub>rest</sub> - 1 =
+ *                    f<sub>rest</sub> / f<sub>obs</sub> - 1).
+ * @param[out] source Pointer to structure to populate.
+ * @return            0 if successful, or 5 if 'name' is too long, else -1 if the 'source'
+ *                    pointer is NULL.
+ *
+ * @sa make_redshifted_object()
+ * @sa novas_v2z()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+int make_redshifted_cat_entry(const char *name, double ra, double dec, double z, cat_entry *source) {
+  static const char *fn = "make_redshifted_cat_entry";
+
+  double v =  novas_z2v(z);
+
+  if(isnan(v))
+    return novas_error(-1, EINVAL, fn, "invalid redshift value: %f\n", z);
+
+  prop_error(fn, make_cat_entry(name, "EXT", 0, ra, dec, 0.0, 0.0, 0.0, v, source), 0);
+  return 0;
+}
+
+/**
+ * Populates a celestial object data structure with the parameters for a redhifted catalog
+ * source, such as a distant quasar or galaxy. It is similar to `make_cat_object()` except
+ * that it takes a Doppler-shift (z) instead of radial velocity and it assumes no parallax
+ * and no proper motion (appropriately for a distant redshifted source). The catalog name
+ * is set to `EXT` to indicate an extragalactic source, and the catalog number defaults to 0.
+ * The user may change these default field values as appropriate afterwards, if necessary.
+ *
+ * @param name        Object name (less than SIZE_OF_OBJ_NAME in length). It may be NULL.
+ * @param ra          [h] Right ascension of the object (hours).
+ * @param dec         [deg] Declination of the object (degrees).
+ * @param z           Redhift value (&lambda;<sub>obs</sub> / &lambda;<sub>rest</sub> - 1 =
+ *                    f<sub>rest</sub> / f<sub>obs</sub> - 1).
+ * @param[out] source Pointer to structure to populate.
+ * @return            0 if successful, or 5 if 'name' is too long, else -1 if the 'source'
+ *                    pointer is NULL.
+ *
+ * @sa make_redshifted_cat_object()
+ * @sa novas_v2z()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+int make_redshifted_object(const char *name, double ra, double dec, double z, object *source) {
+  static const char *fn = "make_redshifted_source";
+
+  cat_entry c;
+
+  prop_error(fn, make_redshifted_cat_entry(name, ra, dec, z, &c), 0);
+  prop_error(fn, make_cat_object(&c, source), 0);
+  return 0;
+}
+
 /**
  * Populates an 'observer' data structure for an observer moving relative to the surface of Earth,
  * such as an airborne observer. Airborne observers have an earth fixed momentary location,
@@ -1082,4 +1211,240 @@ int make_solar_system_observer(const double *sc_pos, const double *sc_vel, obser
   return 0;
 }
 
+/**
+ * Converts a radial recession velocity to a redshift value (z = &delta;f / f<sub>rest</sub>).
+ * It is based on the relativistic formula:
+ * <pre>
+ *  1 + z = sqrt((1 + &beta;) / (1 - &beta;))
+ * </pre>
+ * where &beta; = v / c.
+ *
+ * @param vel   [km/s] velocity (i.e. rate) of recession.
+ * @return      the corresponding redshift value (&delta;&lambda; / &lambda;<sub>rest</sub>), or NAN if
+ *              the input velocity is invalid (i.e., it exceeds the speed of light).
+ *
+ * @sa novas_z2v()
+ * @sa novas_z_add()
+ *
+ * @author Attila Kovacs
+ * @since 1.2
+ */
+double novas_v2z(double vel) {
+  vel *= NOVAS_KM / C;   // [km/s] -> beta
+  if(fabs(vel) > 1.0) {
+    novas_error(-1, EINVAL, "novas_v2z", "velocity exceeds speed of light v=%g km/s", vel * C / NOVAS_KM);
+    return NAN;
+  }
+  return sqrt((1.0 + vel) / (1.0 - vel)) - 1.0;
+}
 
+/**
+ * Returns the gravitational redshift (_z_) for light emitted near a massive spherical body
+ * at some distance from its center, and observed at some very large (infinite) distance away.
+ *
+ * @param M_kg    [kg] Mass of gravitating body that is contained inside the emitting radius.
+ * @param r_m     [m] Radius at which light is emitted.
+ * @return        The gravitational redshift (_z_) for an observer at very large  (infinite)
+ *                distance from the gravitating body.
+ *
+ * @sa redshift_vrad()
+ * @sa unredshift_vrad()
+ * @sa novas_z_add()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+double grav_redshift(double M_kg, double r_m) {
+  static const double twoGoverC2 = 2.0 * 6.6743e-11 / (C * C); // 2G/c^2 in SI units.
+  return 1.0 / sqrt(1.0 - twoGoverC2 * M_kg / r_m) - 1.0;
+}
+
+/**
+ * Applies an incremental redshift correction to a radial velocity. For example, you may
+ * use this function to correct a radial velocity calculated by `rad_vel()` or `rad_vel2()`
+ * for a Solar-system body to account for the gravitational redshift for light originating
+ * at a specific distance away from the body. For the Sun, you may want to undo the redshift
+ * correction applied for the photosphere using `unredshift_vrad()` first.
+ *
+ * @param vrad    [km/s] Radial velocity
+ * @param z       Redshift correction to apply
+ * @return        [km/s] The redshift corrected radial velocity or NAN if the redshift value
+ *                is invalid (errno will be set to EINVAL).
+ *
+ * @sa unredshift_vrad()
+ * @sa grav_redshift()
+ * @sa novas_z_add()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+double redshift_vrad(double vrad, double z) {
+  static const char *fn = "redshift_vrad";
+  double z0;
+  if(z <= -1.0) {
+    novas_error(-1, EINVAL, fn, "invalid redshift value: z=%g", z);
+    return NAN;
+  }
+  z0 = novas_v2z(vrad);
+  if(isnan(z0)) novas_trace(fn, -1, 0);
+  return novas_z2v((1.0 + z0) * (1.0 + z) - 1.0);
+}
+
+/**
+ * Undoes an incremental redshift correction that was applied to radial velocity.
+ *
+ * @param vrad    [km/s] Radial velocity
+ * @param z       Redshift correction to apply
+ * @return        [km/s] The radial velocity without the redshift correction or NAN if the
+ *                redshift value is invalid. (errno will be set to EINVAL)
+ *
+ * @sa redshift_vrad()
+ * @sa grav_redshift()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+double unredshift_vrad(double vrad, double z) {
+  static const char *fn = "unredshift_vrad";
+  double z0;
+  if(z <= -1.0) {
+    novas_error(-1, EINVAL, fn, "invalid redshift value: z=%g", z);
+    return NAN;
+  }
+  z0 = novas_v2z(vrad);
+  if(isnan(z0)) novas_trace(fn, -1, 0);
+  return novas_z2v((1.0 + z0) / (1.0 + z) - 1.0);
+}
+
+/**
+ * Compounds two redshift corrections, e.g. to apply (or undo) a series gravitational redshift
+ * corrections and/or corrections for a moving observer. It's effectively using
+ * (1 + z) = (1 + z1) * (1 + z2).
+ *
+ * @param z1    One of the redshift values
+ * @param z2    The other redshift value
+ * @return      The compound redshift value, ot NAN if either input redshift is invalid (errno
+ *              will be set to EINVAL).
+ *
+ * @sa grav_redshift()
+ * @sa redshift_vrad()
+ * @sa unredshift_vrad()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+double novas_z_add(double z1, double z2) {
+  if(z1 <= -1.0 || z2 <= -1.0) {
+    novas_error(-1, EINVAL, "novas_z_add", "invalid redshift value: z1=%g, z2=%g", z1, z2);
+    return NAN;
+  }
+  return z1 + z2 + z1 * z2;
+}
+
+/**
+ * Returns the inverse of a redshift value, that is the redshift for a body moving with the same
+ * velocity as the original but in the opposite direction.
+ *
+ * @param z     A redhift value
+ * @return      The redshift value for a body moving in the opposite direction with the
+ *              same speed, or NAN if the input redshift is invalid.
+ *
+ * @sa novas_z_add()
+ *
+ * @since 1.2
+ * @author Attila Kovacs
+ */
+double novas_z_inv(double z) {
+  if(z <= -1.0) {
+    novas_error(-1, EINVAL, "novas_z_inv", "invalid redshift value: z=%g", z);
+    return NAN;
+  }
+  return 1.0 / (1.0 + z) - 1.0;
+}
+
+/**
+ * Returns the NOVAS planet ID for a given name (case insensitive), or -1 if no match is found.
+ *
+ * @param name    The planet name, or that for the "Sun", "Moon" or "SSB" (case insensitive).
+ *                The spelled out "Solar System Barycenter" is also recognized with either spaces,
+ *                hyphens ('-') or underscores ('_') separating the case insensitive words.
+ * @return        The NOVAS major planet ID, or -1 (errno set to EINVAL) if the input name is
+ *                NULL or if there is no match for the name provided.
+ *
+ * @author Attila Kovacs
+ * @since 1.2
+ *
+ * @sa make_planet()
+ */
+enum novas_planet novas_planet_for_name(const char *name) {
+  static const char *fn = "novas_planet_for_name()";
+  static const char *names[] = NOVAS_PLANET_NAMES_INIT;
+
+  char *tok;
+  int i;
+
+  if(!name)
+    return novas_error(-1, EINVAL, fn, "Input name is NULL");
+
+  if(!name[0])
+    return novas_error(-1, EINVAL, fn, "Input name is empty");
+
+  for(i = 0; i < NOVAS_PLANETS; i++)
+    if(strcasecmp(name, (const char *) names[i]) == 0)
+      return i;
+
+  // Check for Solar System Barycenter (and variants)
+  tok = strtok(strdup(name), " \t-_");
+  if(strcasecmp("solar", tok) == 0) {
+    tok = strtok(NULL, " \t-_");
+    if(tok && strcasecmp("system", tok) == 0) {
+      tok = strtok(NULL, " \t-_");
+      if(tok && strcasecmp("barycenter", tok) == 0)
+        return NOVAS_SSB;
+    }
+  }
+
+  return novas_error(-1, EINVAL, fn, "No match for name: '%s'", name);
+}
+
+/**
+ * Sets the orientation of an orbital system using the RA and DEC coordinates of the pole
+ * of the Laplace (or else equatorial) plane relative to which the orbital elements are
+ * defined. Orbital parameters of planetary satellites normally include the R.A. and
+ * declination of the pole of the local Laplace plane in which the Keplerian orbital elements
+ * are referenced.
+ *
+ * The system will become referenced to the equatorial plane, the relative obliquity is set
+ * to (90&deg; - `dec`), while the argument of the ascending node ('Omega') is set to
+ * (90&deg; + `ra`).
+ *
+ * NOTES:
+ * <ol>
+ * <li>You should not expect much precision from the long-range orbital approximations for
+ * planetary satellites. For applications that require precision at any level, you should rely
+ * on appropriate ephemerides, or else on up-to-date short-term orbital elements.</li>
+ * </ol>
+ *
+ * @param type  Coordinate reference system in which `ra` and `dec` are defined (e.g. NOVAS_GCRS).
+ * @param ra    [h] the R.A. of the pole of the oribtal reference plane.
+ * @param dec   [deg] the declination of the pole of the oribtal reference plane.
+ * @param[out]  sys   The orbital system
+ * @return      0 if successful, or else -1 (errno will be set to EINVAL) if the output `sys`
+ *              pointer is NULL.
+ *
+ * @author Attila Kovacs
+ * @since 1.2
+ *
+ * @sa make_orbital_object()
+ */
+int novas_set_orbsys_pole(enum novas_reference_system type, double ra, double dec, novas_orbital_system *sys) {
+  if (!sys)
+    return novas_error(-1, EINVAL, "novas_set_orbsys_pole", "input system is NULL");
+
+  sys->plane = NOVAS_EQUATORIAL_PLANE;
+  sys->type = type;
+  sys->obl = remainder(90.0 - dec, 360.0);
+  sys->Omega = remainder(15.0 * ra + 90.0, 360.0);
+
+  return 0;
+}
