@@ -1,6 +1,4 @@
 /**
- * @file
- *
  * @date Created  on Feb 19, 2024
  * @author Attila Kovacs
  */
@@ -12,6 +10,7 @@
 #include <math.h>
 #include <string.h>
 
+#define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
 #include "novas.h"
 
 static int check(const char *func, int exp, int error) {
@@ -144,6 +143,29 @@ static int test_make_cat_object() {
   return n;
 }
 
+static int test_make_redshifted_object() {
+  object source;
+  int n = 0;
+
+  if(check("make_redshifted_object", -1, make_redshifted_object("TEST", 0.0, 0.0, 0.0, NULL))) n++;
+  if(check("make_redshifted_object:z:lo", -1, make_redshifted_object("TEST", 0.0, 0.0, -1.0, &source))) n++;
+
+  return n;
+}
+
+static int test_v2z() {
+  int n = 0;
+
+  if(check_nan("v2z:hi", novas_v2z(NOVAS_C / 1000.0 + 0.01))) n++;
+  return n;
+}
+
+static int test_z2v() {
+  int n = 0;
+
+  if(check_nan("z2v:-1", novas_z2v(-1.0))) n++;
+  return n;
+}
 
 static int test_refract() {
   on_surface o = {};
@@ -255,8 +277,10 @@ static int test_ephemeris() {
   if(check("ephemeris:pos+vel", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, NULL, NULL))) n++;
   if(check("ephemeris:pos=vel", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, p, p))) n++;
   if(check("ephemeris:origin", 1, ephemeris(tdb, &ceres, -1, NOVAS_FULL_ACCURACY, p, v))) n++;
-  if(check("ephemeris:noephem", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, p, v))) n++;
 
+#ifndef DEFAULT_READEPH
+  if(check("ephemeris:noephem", -1, ephemeris(tdb, &ceres, NOVAS_BARYCENTER, NOVAS_FULL_ACCURACY, p, v))) n++;
+#endif
 
   return n;
 }
@@ -283,6 +307,48 @@ static int test_tod_to_j2000() {
   return n;
 }
 
+static int test_gcrs_to_tod() {
+  double p[3];
+  int n = 0;
+
+  if(check("gcrs_to_tod:in", -1, gcrs_to_tod(0.0, NOVAS_FULL_ACCURACY, NULL, p))) n++;
+  if(check("gcrs_to_tod:out", -1, gcrs_to_tod(0.0, NOVAS_FULL_ACCURACY, p, NULL))) n++;
+  if(check("gcrs_to_tod:accuracy", -1, gcrs_to_tod(0.0, -1, p, p))) n++;
+
+  return n;
+}
+
+static int test_tod_to_gcrs() {
+  double p[3] = {1.0};
+  int n = 0;
+
+  if(check("tod_to_gcrs:in", -1, tod_to_gcrs(0.0, NOVAS_FULL_ACCURACY, NULL, p))) n++;
+  if(check("tod_to_gcrs:out", -1, tod_to_gcrs(0.0, NOVAS_FULL_ACCURACY, p, NULL))) n++;
+  if(check("tod_to_gcrs:accuracy", -1, tod_to_gcrs(0.0, -1, p, p))) n++;
+
+  return n;
+}
+
+static int test_gcrs_to_mod() {
+  double p[3];
+  int n = 0;
+
+  if(check("gcrs_to_mod:in", -1, gcrs_to_mod(0.0, NULL, p))) n++;
+  if(check("gcrs_to_mod:out", -1, gcrs_to_mod(0.0, p, NULL))) n++;
+
+  return n;
+}
+
+static int test_mod_to_gcrs() {
+  double p[3] = {1.0};
+  int n = 0;
+
+  if(check("mod_to_gcrs:in", -1, mod_to_gcrs(0.0, NULL, p))) n++;
+  if(check("mod_to_gcrs:out", -1, mod_to_gcrs(0.0, p, NULL))) n++;
+
+  return n;
+}
+
 static int test_gcrs_to_cirs() {
   double p[3] = {1.0};
   int n = 0;
@@ -1052,6 +1118,10 @@ static int test_time() {
 
 static double switching_refraction(double jd_tt, const on_surface *loc, enum novas_refraction_type type, double el) {
   static int count;
+  (void) jd_tt;
+  (void) loc;
+  (void) type;
+  (void) el;
   return (count++) % 2 ? -0.1 : 0.1;
 }
 
@@ -1083,6 +1153,12 @@ static int test_refraction() {
   novas_optical_refraction(NOVAS_JD_J2000, NULL, NOVAS_REFRACT_OBSERVED, 10.0);
   novas_debug(0);
 
+  obs.humidity = -1.01;
+  if(check_nan("radio_refraction:humidity:lo", novas_radio_refraction(NOVAS_JD_J2000, &obs, NOVAS_REFRACT_OBSERVED, 10.0))) n++;
+
+  obs.humidity = 101.01;
+  if(check_nan("radio_refraction:humidity:hi", novas_radio_refraction(NOVAS_JD_J2000, &obs, NOVAS_REFRACT_OBSERVED, 10.0))) n++;
+
   return n;
 }
 
@@ -1125,7 +1201,7 @@ static int test_change_observer() {
   if(check("change_observer:obs", -1, novas_change_observer(&frame, NULL, &out))) n++;
   if(check("change_observer:out", -1, novas_change_observer(&frame, &obs, NULL))) n++;
 
-  return 0;
+  return n;
 }
 
 static int test_make_transform() {
@@ -1182,9 +1258,11 @@ static int test_geom_posvel() {
   frame.accuracy = 2;
   if(check("geom_posvel:frame:accuracy:2", -1, novas_geom_posvel(&o, &frame, NOVAS_ICRS, pos, vel))) n++;
 
+#ifndef DEFAULT_READEPH
   frame.accuracy = NOVAS_REDUCED_ACCURACY;
   make_ephem_object("blah", 111111, &o);
   if(check("geom_posvel:ephem_object", -1, novas_geom_posvel(&o, &frame, NOVAS_ICRS, pos, vel))) n++;
+#endif
 
   return n;
 }
@@ -1389,9 +1467,148 @@ static int test_inv_transform() {
   return n;
 }
 
+static int test_redshift_vrad() {
+  int n = 0;
+
+  if(check_nan("redshift_vrad", redshift_vrad(0.0, -1.0))) n++;
+  return n;
+}
+
+static int test_unredshift_vrad() {
+  int n = 0;
+
+  if(check_nan("unredshift_vrad", unredshift_vrad(0.0, -1.0))) n++;
+  return n;
+}
+
+static int test_z_add() {
+  int n = 0;
+
+  if(check_nan("z_add:z1", novas_z_add(-1.0, 0.0))) n++;
+  if(check_nan("z_add:z2", novas_z_add(0.0, -1.0))) n++;
+  if(check_nan("z_add:z1+z2", novas_z_add(-1.0, -1.0))) n++;
+  return n;
+}
+
+static int test_z_inv() {
+  int n = 0;
+
+  if(check_nan("z_inv", novas_z_inv(-1.0))) n++;
+  return n;
+}
+
+static int test_novas_to_naif_planet() {
+  int n = 0;
+
+  if(check("novas_to_naif_planet:-2", -1, novas_to_naif_planet(-2))) n++;
+  if(check("novas_to_naif_planet:-1", -1, novas_to_naif_planet(-1))) n++;
+  if(check("novas_to_naif_planet:hi", -1, novas_to_naif_planet(NOVAS_PLANETS))) n++;
+
+  return n;
+}
+
+static int test_novas_to_dexxx_planet() {
+  int n = 0;
+
+  if(check("novas_to_dexxx_planet:-2", -1, novas_to_dexxx_planet(-2))) n++;
+  if(check("novas_to_dexxx_planet:-1", -1, novas_to_dexxx_planet(-1))) n++;
+  if(check("novas_to_dexxx_planet:hi", -1, novas_to_dexxx_planet(NOVAS_PLANETS))) n++;
+
+  return n;
+}
+
+static int test_naif_to_novas_planet() {
+  int n = 0;
+
+  if(check("naif_to_novas_planet:-2", -1, naif_to_novas_planet(-1))) n++;
+  if(check("naif_to_novas_planet:-1", -1, naif_to_novas_planet(-1))) n++;
+  if(check("naif_to_novas_planet:", -1, naif_to_novas_planet(-1))) n++;
+  if(check("naif_to_novas_planet:500", -1, naif_to_novas_planet(500))) n++;
+  if(check("naif_to_novas_planet:501", -1, naif_to_novas_planet(501))) n++;
+  if(check("naif_to_novas_planet:598", -1, naif_to_novas_planet(598))) n++;
+  if(check("naif_to_novas_planet:1000", -1, naif_to_novas_planet(1000))) n++;
+
+  return n;
+}
+
+static int test_planet_for_name() {
+  int n = 0;
+
+  if(check("planet_for_name:NULL", -1, novas_planet_for_name(NULL))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name(""))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name("blah"))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name("solar"))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name("Solar flare"))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name("Solar system"))) n++;
+  if(check("planet_for_name:blah", -1, novas_planet_for_name("Solar system size"))) n++;
+
+  return n;
+}
+
+static int test_make_orbital_object() {
+  int n = 0;
+  novas_orbital orbit = {};
+  object body = {};
+
+  if(check("make_orbital_object:orbit", -1, make_orbital_object("blah", -1, NULL, &body))) n++;
+  if(check("make_orbital_object:body", -1, make_orbital_object("blah", -1, &orbit, NULL))) n++;
+  if(check("make_orbital_object:orbit+body", -1, make_orbital_object("blah", -1, NULL, NULL))) n++;
+
+  return n;
+}
+
+static int test_orbit_posvel() {
+  int n = 0;
+  double pos[3] = {}, vel[3] = {};
+  int saved = novas_inv_max_iter;
+  novas_orbital orbit = NOVAS_ORBIT_INIT;
+
+  orbit.a = 1.0;
+
+  if(check("set_obsys_pole:orbit", -1, novas_orbit_posvel(0.0, NULL, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+  if(check("set_obsys_pole:pos=vel", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, pos))) n++;
+  if(check("set_obsys_pole:pos=vel:NULL", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, NULL, NULL))) n++;
+  if(check("set_obsys_pole:accuracy:-1", -1, novas_orbit_posvel(0.0, &orbit, -1, pos, vel))) n++;
+  if(check("set_obsys_pole:accuracy:2", -1, novas_orbit_posvel(0.0, &orbit, 2, pos, vel))) n++;
+
+  if(check("set_obsys_pole:orbit:converge", 0, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  novas_inv_max_iter = 0;
+  if(check("set_obsys_pole:orbit:converge", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+  else if(check("set_obsys_pole:orbit:converge:errno", ECANCELED, errno)) n++;
+  novas_inv_max_iter = saved;
+
+  orbit.system.type = -1;
+  if(check("set_obsys_pole:orbit:type:-1", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  orbit.system.type = NOVAS_REFERENCE_SYSTEMS;
+  if(check("set_obsys_pole:orbit:type:hi", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  orbit.system.plane = NOVAS_EQUATORIAL_PLANE;
+  orbit.system.type = NOVAS_REFERENCE_SYSTEMS;
+  if(check("set_obsys_pole:orbit:type:-1:eq", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  orbit.system.type = NOVAS_GCRS;
+  orbit.system.plane = -1;
+  if(check("set_obsys_pole:orbit:plane:-1", -1, novas_orbit_posvel(0.0, &orbit, NOVAS_REDUCED_ACCURACY, pos, vel))) n++;
+
+  return n;
+}
+
+static int test_set_obsys_pole() {
+  int n = 0;
+
+  if(check("set_obsys_pole:orbit", -1, novas_set_orbsys_pole(NOVAS_GCRS, 0.0, 0.0, NULL))) n++;
+
+  return n;
+}
+
 int main() {
   int n = 0;
 
+  if(test_v2z()) n++;
+  if(test_z2v()) n++;
+
   if(test_make_on_surface()) n++;
   if(test_make_in_space()) n++;
   if(test_make_observer()) n++;
@@ -1399,6 +1616,7 @@ int main() {
 
   if(test_make_object()) n++;
   if(test_make_cat_object()) n++;
+  if(test_make_redshifted_object()) n++;
   if(test_make_ephem_object()) n++;
   if(test_make_planet()) n++;
   if(test_make_cat_entry()) n++;
@@ -1504,6 +1722,24 @@ int main() {
   if(test_transform_sky_pos()) n++;
   if(test_inv_transform()) n++;
 
+  if(test_redshift_vrad()) n++;
+  if(test_unredshift_vrad()) n++;
+  if(test_z_add()) n++;
+  if(test_z_inv()) n++;
+
+  if(test_novas_to_naif_planet()) n++;
+  if(test_novas_to_dexxx_planet()) n++;
+  if(test_naif_to_novas_planet()) n++;
+
+  if(test_planet_for_name()) n++;
+  if(test_make_orbital_object()) n++;
+  if(test_set_obsys_pole()) n++;
+  if(test_orbit_posvel()) n++;
+
+  if(test_gcrs_to_tod()) n++;
+  if(test_tod_to_gcrs()) n++;
+  if(test_gcrs_to_mod()) n++;
+  if(test_mod_to_gcrs()) n++;
 
   if(n) fprintf(stderr, " -- FAILED %d tests\n", n);
   else fprintf(stderr, " -- OK\n");
