@@ -1,11 +1,17 @@
 /**
- * @file
- *
  * @date Created  on Feb 18, 2024
  * @author Attila Kovacs
  */
 
 
+// We'll use gcc major version as a proxy for the glibc library to decide which feature macro to use.
+// gcc 5.1 was released 2015-04-22...
+#if defined(__GNUC__) && (__GNUC__ < 5)
+#  define _BSD_SOURCE            ///< strcasecmp() feature macro for glibc <= 2.19
+#else
+#  define _DEFAULT_SOURCE        ///< strcasecmp() feature macro starting glibc 2.20 (2014-09-08)
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -18,8 +24,8 @@
 #define __NOVAS_INTERNAL_API__      ///< Use definitions meant for internal use by SuperNOVAS only
 #include "novas.h"
 
-#define J2000   2451545.0
-#define DAY     86400.0
+#define J2000   NOVAS_JD_J2000
+
 
 static char *workPath;
 
@@ -37,6 +43,8 @@ static double pos0[3];
 static enum novas_origin ephem_origin;
 
 static short dummy_planet_hp(const double *jd_tdb, enum novas_planet body, enum novas_origin origin, double *position, double *velocity) {
+  (void) jd_tdb;
+  (void) origin;
   memset(position, 0, 3 * sizeof(double));
   memset(velocity, 0, 3 * sizeof(double));
   position[0] = body % 10;
@@ -46,10 +54,14 @@ static short dummy_planet_hp(const double *jd_tdb, enum novas_planet body, enum
 
 static short dummy_planet(double jd_tdb, enum novas_planet body, enum novas_origin origin, double *position, double *velocity) {
   double tdb2[2] = { tdb };
+  (void) jd_tdb;
   return dummy_planet_hp(tdb2, body, origin, position, velocity);
 }
 
 static int dummy_ephem(const char *name, long id, double jd_tdb_high, double jd_tdb_low, enum novas_origin *origin, double *pos, double *vel) {
+  (void) name;
+  (void) jd_tdb_high;
+  (void) jd_tdb_low;
   *origin = ephem_origin;
   memset(pos, 0, 3 * sizeof(double));
   memset(vel, 0, 3 * sizeof(double));
@@ -68,7 +80,7 @@ static int check_equal_pos(const double *posa, const double *posb, double tol) {
     if(fabs(posa[i] - posb[i]) <= tol) continue;
     if(isnan(posa[i]) && isnan(posb[i])) continue;
 
-    fprintf(stderr, "  A[%d] = %.9g vs B[%d] = %.9g\n", i, posa[i], i, posb[i]);
+    fprintf(stderr, "  A[%d] = %.9g vs B[%d] = %.9g (delta=%.1g)\n", i, posa[i], i, posb[i], posa[i] - posb[i]);
     return i + 1;
   }
 
@@ -736,7 +748,43 @@ static int test_transform_inv() {
   return 0;
 }
 
+static int test_gcrs_to_tod() {
+  double pos1[3] = {}, pos2[3] = {}, d;
+  int n = 0;
 
+  d = novas_vlen(pos0);
+
+  if(!is_ok("gcrs_to_tod", gcrs_to_tod(tdb, NOVAS_FULL_ACCURACY, pos0, pos1))) n++;
+
+  gcrs_to_j2000(pos0, pos2);
+  j2000_to_tod(tdb, NOVAS_FULL_ACCURACY, pos2, pos2);
+
+  if(!is_ok("gcrs_to_tod:check", check_equal_pos(pos1, pos2, 1e-9 * d))) n++;
+
+  if(!is_ok("gcrs_to_tod:tod_to_gcrs", tod_to_gcrs(tdb, NOVAS_FULL_ACCURACY, pos1, pos2))) n++;
+  if(!is_ok("gcrs_to_tod:tod_to_gcrs:check", check_equal_pos(pos2, pos0, 1e-9 * d))) n++;
+
+  return n;
+}
+
+static int test_gcrs_to_mod() {
+  double pos1[3] = {}, pos2[3] = {}, d;
+  int n = 0;
+
+  d = novas_vlen(pos0);
+
+  if(!is_ok("gcrs_to_mod", gcrs_to_mod(tdb, pos0, pos1))) n++;
+
+  gcrs_to_j2000(pos0, pos2);
+  precession(NOVAS_JD_J2000, pos2, tdb, pos2);
+
+  if(!is_ok("gcrs_to_mod:check", check_equal_pos(pos1, pos2, 1e-9 * d))) n++;
+
+  if(!is_ok("gcrs_to_mod:mod_to_gcrs", mod_to_gcrs(tdb, pos1, pos2))) n++;
+  if(!is_ok("gcrs_to_mod:mod_to_gcrs:check", check_equal_pos(pos2, pos0, 1e-9 * d))) n++;
+
+  return n;
+}
 
 static int test_source() {
   int k, n = 0;
@@ -774,6 +822,9 @@ static int test_source() {
   if(test_transform_mod_tod()) n++;
   if(test_transform_inv()) n++;
 
+  if(test_gcrs_to_tod()) n++;
+  if(test_gcrs_to_mod()) n++;
+
   for(k = 0; k < NOVAS_REFERENCE_SYSTEMS; k++)  if(test_app_hor(k)) n++;
   for(k = 0; k < NOVAS_REFERENCE_SYSTEMS; k++)  if(test_app_geom(k)) n++;
 
@@ -784,7 +835,7 @@ static int test_source() {
 static int test_make_planet() {
   object mars;
 
-  make_planet(NOVAS_MARS, &mars);
+  if(!is_ok("make_panet", make_planet(NOVAS_MARS, &mars))) return 1;
 
   if(!is_ok("make_planet:type", mars.type != NOVAS_PLANET)) return 1;
   if(!is_ok("make_planet:number", mars.number != NOVAS_MARS)) return 1;
@@ -1260,6 +1311,23 @@ static int test_refract_astro() {
   return 0;
 }
 
+static int test_v2z() {
+  int v;
+
+  for(v = 0.0; v < NOVAS_C; v += 10000000) {
+    char label[40];
+    double zexp = sqrt((1.0 + v / NOVAS_C) / (1.0 - v / NOVAS_C)) - 1.0;
+
+    sprintf(label, "v2z:v:%d", v);
+    if(!is_equal(label, novas_v2z(v / 1000.0), zexp, 1e-6)) return 1;
+
+    sprintf(label, "v2z:z2v:v:%d", v);
+    if(!is_equal(label, novas_z2v(zexp), v / 1000.0, 1e-6)) return 1;
+  }
+
+  return 0;
+}
+
 static int test_case() {
   object o;
 
@@ -1307,6 +1375,22 @@ static int test_make_object() {
   return 0;
 }
 
+static int test_make_redshifted_object() {
+  object gal;
+
+  if(!is_ok("make_redshifted_object", make_redshifted_object("test", 1.0, 2.0, 3.0, &gal))) return 1;
+
+  if(!is_ok("make_redshifted_object:type", gal.type != NOVAS_CATALOG_OBJECT)) return 1;
+  if(!is_equal("make_redshifted_object:ra", gal.star.ra, 1.0, 1e-12)) return 1;
+  if(!is_equal("make_redshifted_object:dec", gal.star.dec, 2.0, 1e-12)) return 1;
+  if(!is_equal("make_redshifted_object:rv", novas_v2z(gal.star.radialvelocity), 3.0, 1e-12)) return 1;
+  if(!is_ok("make_redshifted_object:ra", gal.star.promora != 0.0)) return 1;
+  if(!is_ok("make_redshifted_object:ra", gal.star.promodec != 0.0)) return 1;
+  if(!is_ok("make_redshifted_object:ra", gal.star.parallax != 0.0)) return 1;
+
+  return 0;
+}
+
 static int test_transform_cat() {
   cat_entry in = {}, out;
 
@@ -1325,7 +1409,10 @@ static int test_planet_provider() {
   make_planet(NOVAS_MARS, &mars);
 
   if(!is_ok("planet_provider:set_planet_provider", set_planet_provider(dummy_planet))) goto cleanup; // @suppress("Goto statement used")
+  if(!is_ok("planet_provider:get_planet_provider", get_planet_provider() != dummy_planet)) goto cleanup; // @suppress("Goto statement used")
+
   if(!is_ok("planet_provider:set_planet_provider_hp", set_planet_provider_hp(dummy_planet_hp))) goto cleanup; // @suppress("Goto statement used")
+  if(!is_ok("planet_provider:get_planet_provider_hp", get_planet_provider_hp() != dummy_planet_hp)) goto cleanup; // @suppress("Goto statement used")
 
   if(!is_ok("planet_provider:ephemeris", ephemeris(tdb2, &mars, NOVAS_BARYCENTER, NOVAS_REDUCED_ACCURACY, p, v))) goto cleanup; // @suppress("Goto statement used")
   if(!is_ok("planet_provider:control", dummy_planet(tdb, NOVAS_MARS, NOVAS_BARYCENTER, p0, v0))) goto cleanup; // @suppress("Goto statement used")
@@ -1557,7 +1644,7 @@ static int test_airborne_observer() {
 
 static int test_solar_system_observer() {
   observer obs = {}, gc = {};
-  object earth = { NOVAS_PLANET, NOVAS_EARTH, "Earth"};
+  object earth = NOVAS_EARTH_INIT;
   double pos[3] = {1.0, 2.0, 3.0}, vel[3] = { 10.0, 11.0, 12.0 };
   double epos[3], evel[3], gpos[3], gvel[3], opos[3], ovel[3];
   double tdb2[2] = { tdb, 0.0 };
@@ -1596,7 +1683,7 @@ static int test_solar_system_observer() {
 static int test_obs_posvel() {
   double epos[3] = {}, evel[3] = {}, x[3];
   observer obs;
-  object earth = { NOVAS_PLANET, NOVAS_EARTH, "Earth"};
+  object earth = NOVAS_EARTH_INIT;
   double tdb2[2] = { tdb, 0.0 };
   double sc_pos[3] = {1.0, 2.0, 3.0}, sc_vel[3] = {4.0, 5.0, 6.0};
   double gpos[3], gvel[3];
@@ -1707,7 +1794,7 @@ static int test_unix_time() {
     printf("!!! sec: %ld  %ld\n", (long) novas_get_unix_time(&t, &nsec), sec);
     return 1;
   }
-  if(!is_ok("sunix_time:check:nsec", abs(nsec - nanos) > 0)) {
+  if(!is_ok("sunix_time:check:nsec", labs(nsec - nanos) > 0)) {
     printf("!!! nsec %ld  %ld\n", nsec, nanos);
     return 1;
   }
@@ -1724,7 +1811,7 @@ static int test_unix_time() {
     printf("!!! sec: %ld  %ld\n", (long) novas_get_unix_time(&t, &nsec), (long) sec);
     return 1;
   }
-  if(!is_ok("unix_time:offset:check:incr:nsec", abs(nsec - nanos) > 0)) {
+  if(!is_ok("unix_time:offset:check:incr:nsec", labs(nsec - nanos) > 0)) {
     printf("!!! nsec %ld  %ld\n", nsec, nanos);
     return 1;
   }
@@ -1735,7 +1822,7 @@ static int test_unix_time() {
     printf("!!! sec: %ld  %ld\n", (long) novas_get_unix_time(&t, &nsec), (long) sec);
     return 1;
   }
-  if(!is_ok("unix_time:neg:check:nsec", abs(nsec - nanos) > 0)) {
+  if(!is_ok("unix_time:neg:check:nsec", labs(nsec - nanos) > 0)) {
     printf("!!! nsec %ld  %ld\n", nsec, nanos);
     return 1;
   }
@@ -2015,9 +2102,331 @@ static int test_rad_vel2() {
   return n;
 }
 
+
+static int test_grav_redshift() {
+  static const double G = 6.6743e-11; // G in SI units.
+  static const double c2 = C * C;
+
+  double M = 2e30;  // [kg]
+  double r = NOVAS_SOLAR_RADIUS;
+
+  double rs = 2 * G * M / c2;
+  double zp1 = 1.0 / sqrt(1.0 - rs / r);
+
+  int n = 0;
+
+  if(!is_equal("grav_redshift", 1.0 + grav_redshift(M, r), zp1, 1e-12)) n++;
+
+  return n;
+}
+
+static int test_redshift_vrad() {
+  double v0 = 100.0;
+  double z;
+
+  int n = 0;
+
+  for(z = -0.5; z < 3.0; z += 0.1) {
+    char label[80];
+    double v = redshift_vrad(v0, z);
+
+    sprintf(label, "redshift_vrad:z=%.1f:inv", z);
+    if(!is_equal(label, unredshift_vrad(v, z), v0, 1e-6)) n++;
+
+    sprintf(label, "redshift_vrad:z=%.1f:check", z);
+    if(!is_equal(label, 1.0 + novas_v2z(v), (1.0 + novas_v2z(v0)) * (1.0 + z), 1e-6)) n++;
+  }
+
+  return n;
+}
+
+static int test_z_add() {
+  int n = 0;
+  double z1;
+
+  for(z1 = -0.5; z1 < 5.0; z1 += 0.5) {
+    double z2;
+    for(z2 = -0.1; z2 < 1.0; z2 += 0.1) {
+      double zexp;
+
+      zexp = (1.0 + z1) * (1.0 + z2) - 1.0;
+      if(!is_equal("z_add", novas_z_add(z1, z2), zexp, 1e-12)) n++;
+    }
+  }
+  return n;
+}
+
+static int test_z_inv() {
+  int n = 0;
+  double z;
+
+  for(z = -0.5; z < 5.0; z += 0.5) {
+    char label[80];
+    double zi = novas_z_inv(z);
+
+    sprintf(label, "z_inv:z=%.1f", z);
+    if(!is_equal(label, 1.0, (1.0 + z) * (1.0 + zi), 1e-6)) n++;
+  }
+
+  return n;
+}
+
+static int test_novas_to_naif_planet() {
+  int n = 0;
+
+  if(!is_ok("novas_to_naif_planet:ssb", novas_to_naif_planet(NOVAS_SSB) != NAIF_SSB)) n++;
+  if(!is_ok("novas_to_naif_planet:sun", novas_to_naif_planet(NOVAS_SUN) != NAIF_SUN)) n++;
+  if(!is_ok("novas_to_naif_planet:moon", novas_to_naif_planet(NOVAS_MOON) != NAIF_MOON)) n++;
+  if(!is_ok("novas_to_naif_planet:earth", novas_to_naif_planet(NOVAS_EARTH) != NAIF_EARTH)) n++;
+  if(!is_ok("novas_to_naif_planet:earth", novas_to_naif_planet(NOVAS_EMB) != NAIF_EMB)) n++;
+  if(!is_ok("novas_to_naif_planet:mercury", novas_to_naif_planet(NOVAS_MERCURY) != 199)) n++;
+  if(!is_ok("novas_to_naif_planet:venus", novas_to_naif_planet(NOVAS_VENUS) != 299)) n++;
+  if(!is_ok("novas_to_naif_planet:mars", novas_to_naif_planet(NOVAS_MARS) != 499)) n++;
+  if(!is_ok("novas_to_naif_planet:jupiter", novas_to_naif_planet(NOVAS_JUPITER) != 599)) n++;
+  if(!is_ok("novas_to_naif_planet:saturn", novas_to_naif_planet(NOVAS_SATURN) != 699)) n++;
+  if(!is_ok("novas_to_naif_planet:uranus", novas_to_naif_planet(NOVAS_URANUS) != 799)) n++;
+  if(!is_ok("novas_to_naif_planet:neptune", novas_to_naif_planet(NOVAS_NEPTUNE) != 899)) n++;
+  if(!is_ok("novas_to_naif_planet:pluto", novas_to_naif_planet(NOVAS_PLUTO) != 999)) n++;
+  if(!is_ok("novas_to_dexxx_planet:pluto", novas_to_naif_planet(NOVAS_PLUTO_BARYCENTER) != 9)) n++;
+
+  return n;
+}
+
+static int test_novas_to_dexxx_planet() {
+  int n = 0;
+
+  if(!is_ok("novas_to_dexxx_planet:ssb", novas_to_dexxx_planet(NOVAS_SSB) != NAIF_SSB)) n++;
+  if(!is_ok("novas_to_dexxx_planet:sun", novas_to_dexxx_planet(NOVAS_SUN) != NAIF_SUN)) n++;
+  if(!is_ok("novas_to_dexxx_planet:moon", novas_to_dexxx_planet(NOVAS_MOON) != NAIF_MOON)) n++;
+  if(!is_ok("novas_to_dexxx_planet:earth", novas_to_dexxx_planet(NOVAS_EARTH) != NAIF_EARTH)) n++;
+  if(!is_ok("novas_to_dexxx_planet:earth", novas_to_dexxx_planet(NOVAS_EMB) != NAIF_EMB)) n++;
+  if(!is_ok("novas_to_dexxx_planet:mercury", novas_to_dexxx_planet(NOVAS_MERCURY) != 1)) n++;
+  if(!is_ok("novas_to_dexxx_planet:venus", novas_to_dexxx_planet(NOVAS_VENUS) != 2)) n++;
+  if(!is_ok("novas_to_dexxx_planet:mars", novas_to_dexxx_planet(NOVAS_MARS) != 4)) n++;
+  if(!is_ok("novas_to_dexxx_planet:jupiter", novas_to_dexxx_planet(NOVAS_JUPITER) != 5)) n++;
+  if(!is_ok("novas_to_dexxx_planet:saturn", novas_to_dexxx_planet(NOVAS_SATURN) != 6)) n++;
+  if(!is_ok("novas_to_dexxx_planet:uranus", novas_to_dexxx_planet(NOVAS_URANUS) != 7)) n++;
+  if(!is_ok("novas_to_dexxx_planet:neptune", novas_to_dexxx_planet(NOVAS_NEPTUNE) != 8)) n++;
+  if(!is_ok("novas_to_dexxx_planet:pluto", novas_to_dexxx_planet(NOVAS_PLUTO) != 9)) n++;
+  if(!is_ok("novas_to_dexxx_planet:pluto", novas_to_dexxx_planet(NOVAS_PLUTO_BARYCENTER) != 9)) n++;
+
+  return n;
+}
+
+static int test_naif_to_novas_planet() {
+  int n = 0;
+
+  if(!is_ok("naif_to_novas_planet:ssb", naif_to_novas_planet(NAIF_SSB) != NOVAS_SSB)) n++;
+  if(!is_ok("naif_to_novas_planet:sun", naif_to_novas_planet(NAIF_SUN) != NOVAS_SUN)) n++;
+  if(!is_ok("naif_to_novas_planet:moon", naif_to_novas_planet(NAIF_MOON) != NOVAS_MOON)) n++;
+  if(!is_ok("naif_to_novas_planet:earth", naif_to_novas_planet(NAIF_EARTH) != NOVAS_EARTH)) n++;
+  if(!is_ok("naif_to_novas_planet:earth", naif_to_novas_planet(NAIF_EMB) != NOVAS_EMB)) n++;
+  if(!is_ok("naif_to_novas_planet:mercury", naif_to_novas_planet(199) != NOVAS_MERCURY)) n++;
+  if(!is_ok("naif_to_novas_planet:venus", naif_to_novas_planet(299) != NOVAS_VENUS)) n++;
+  if(!is_ok("naif_to_novas_planet:mars", naif_to_novas_planet(499) != NOVAS_MARS)) n++;
+  if(!is_ok("naif_to_novas_planet:jupiter", naif_to_novas_planet(599) != NOVAS_JUPITER)) n++;
+  if(!is_ok("naif_to_novas_planet:saturn", naif_to_novas_planet(699) != NOVAS_SATURN)) n++;
+  if(!is_ok("naif_to_novas_planet:uranus", naif_to_novas_planet(799) != NOVAS_URANUS)) n++;
+  if(!is_ok("naif_to_novas_planet:neptune", naif_to_novas_planet(899) != NOVAS_NEPTUNE)) n++;
+  if(!is_ok("naif_to_novas_planet:pluto", naif_to_novas_planet(999) != NOVAS_PLUTO)) n++;
+  if(!is_ok("naif_to_novas_planet:mercury", naif_to_novas_planet(1) != NOVAS_MERCURY)) n++;
+  if(!is_ok("naif_to_novas_planet:venus", naif_to_novas_planet(2) != NOVAS_VENUS)) n++;
+  if(!is_ok("naif_to_novas_planet:mars", naif_to_novas_planet(4) != NOVAS_MARS)) n++;
+  if(!is_ok("naif_to_novas_planet:jupiter", naif_to_novas_planet(5) != NOVAS_JUPITER)) n++;
+  if(!is_ok("naif_to_novas_planet:saturn", naif_to_novas_planet(6) != NOVAS_SATURN)) n++;
+  if(!is_ok("naif_to_novas_planet:uranus", naif_to_novas_planet(7) != NOVAS_URANUS)) n++;
+  if(!is_ok("naif_to_novas_planet:neptune", naif_to_novas_planet(8) != NOVAS_NEPTUNE)) n++;
+  if(!is_ok("naif_to_novas_planet:pluto", naif_to_novas_planet(9) != NOVAS_PLUTO_BARYCENTER)) n++;
+  if(!is_ok("naif_to_novas_planet:pluto", naif_to_novas_planet(999) != NOVAS_PLUTO)) n++;
+
+  return n;
+}
+
+static int test_planet_for_name() {
+  int n = 0;
+
+  if(!is_ok("planet_for_name:mercury", novas_planet_for_name("mercury") != NOVAS_MERCURY)) n++;
+  if(!is_ok("planet_for_name:pluto", novas_planet_for_name("PLUTO") != NOVAS_PLUTO)) n++;
+  if(!is_ok("planet_for_name:sun", novas_planet_for_name("Sun") != NOVAS_SUN)) n++;
+  if(!is_ok("planet_for_name:moon", novas_planet_for_name("MooN") != NOVAS_MOON)) n++;
+  if(!is_ok("planet_for_name:ssb", novas_planet_for_name("SSB") != NOVAS_SSB)) n++;
+  if(!is_ok("planet_for_name:ssb1", novas_planet_for_name("Solar-system barycenter") != NOVAS_SSB)) n++;
+
+  return n;
+}
+
+
+static int test_orbit_place() {
+  object ceres = {};
+  novas_orbital orbit = NOVAS_ORBIT_INIT;
+  observer obs = {};
+  sky_pos pos = {};
+  double p0[3] = {}, p1[3] = {};
+
+  // Nov 14 0 UTC, geocentric from JPL Horizons.
+  double tjd = 2460628.50079861;      // 0 UT as TT.
+  double RA0 = 19.684415;
+  double DEC0 = -28.62084;
+  double rv0 = 21.4255198;            // km/s
+  double r = 3.32557776285144;        // AU
+  int n = 0;
+
+  // Orbital Parameters for JD 2460600.5 from MPC
+  orbit.jd_tdb = 2460600.5;
+  orbit.a = 2.7666197;
+  orbit.e = 0.079184;
+  orbit.i = 10.5879;
+  orbit.omega = 73.28579;
+  orbit.Omega = 80.25414;
+  orbit.M0 = 145.84905;
+  orbit.n = 0.21418047;
+
+  make_observer_at_geocenter(&obs);
+  make_orbital_object("Ceres", -1, &orbit, &ceres);
+
+  if(!is_ok("orbit_place", place(tjd, &ceres, &obs, ut12tt, NOVAS_TOD, NOVAS_REDUCED_ACCURACY, &pos))) return 1;
+
+  if(!is_equal("orbit_place:ra", pos.ra, RA0, 1e-5 / cos(DEC0 * DEGREE))) n++;
+  if(!is_equal("orbit_place:dec", pos.dec, DEC0, 1e-4)) n++;
+  if(!is_equal("orbit_place:dist", pos.dis, r, 1e-4)) n++;
+  if(!is_equal("orbit_place:vrad", pos.rv, rv0, 1e-2)) n++;
+
+  if(!is_ok("orbit_place", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p0, NULL))) return 1;
+  equ2ecl_vec(tjd, NOVAS_GCRS_EQUATOR, NOVAS_FULL_ACCURACY, p0, p0);
+
+  orbit.system.type = NOVAS_ICRS;
+  if(!is_ok("orbit_place:icrs", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p1, NULL))) n++;
+  else {
+    equ2ecl_vec(tjd, NOVAS_GCRS_EQUATOR, NOVAS_FULL_ACCURACY, p1, p1);
+    if(!is_ok("orbit_place:icrs:check", check_equal_pos(p1, p0, 1e-9))) n++;
+  }
+
+  orbit.system.type = NOVAS_CIRS;
+  if(!is_ok("orbit_place:cirs", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p1, NULL))) n++;
+  else {
+    gcrs_to_cirs(tjd, NOVAS_REDUCED_ACCURACY, p1, p1);
+    equ2ecl_vec(tjd, NOVAS_TRUE_EQUATOR, NOVAS_FULL_ACCURACY, p1, p1);
+    if(!is_ok("orbit_place:cirs:check", check_equal_pos(p1, p0, 1e-9))) n++;
+  }
+
+  orbit.system.type = NOVAS_J2000;
+  if(!is_ok("orbit_place:j2000", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p1, NULL))) n++;
+  else {
+    gcrs_to_j2000(p1, p1);
+    equ2ecl_vec(NOVAS_JD_J2000, NOVAS_TRUE_EQUATOR, NOVAS_FULL_ACCURACY, p1, p1);
+    if(!is_ok("orbit_place:j2000:check", check_equal_pos(p1, p0, 1e-9))) n++;
+  }
+
+  orbit.system.type = NOVAS_MOD;
+  if(!is_ok("orbit_place:mod", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p1, NULL))) n++;
+  else {
+    gcrs_to_mod(tjd, p1, p1);
+    equ2ecl_vec(tjd, NOVAS_MEAN_EQUATOR, NOVAS_FULL_ACCURACY, p1, p1);
+    if(!is_ok("orbit_place:mod:check", check_equal_pos(p1, p0, 1e-9))) n++;
+  }
+
+  orbit.system.type = NOVAS_TOD;
+  if(!is_ok("orbit_place:tod", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, p1, NULL))) n++;
+  else {
+    gcrs_to_tod(tjd, NOVAS_FULL_ACCURACY, p1, p1);
+    equ2ecl_vec(tjd, NOVAS_TRUE_EQUATOR, NOVAS_FULL_ACCURACY, p1, p1);
+    if(!is_ok("orbit_place:tod:check", check_equal_pos(p1, p0, 1e-9))) n++;
+  }
+
+  return n;
+}
+
+
+static int test_orbit_posvel_callisto() {
+  novas_orbital orbit = NOVAS_ORBIT_INIT;
+  novas_orbital_system *sys = &orbit.system;
+  double pos0[3] = {}, pos[3] = {}, vel[3] = {}, pos1[3] = {}, vel1[3] = {}, ra, dec, dra, ddec;
+  int i;
+
+  // 2000-01-01 12 UT, geocentric from JPL Horizons.
+
+  double dist = 4.62117513332102;
+  double lt = 0.00577551831217194 * dist;                 // day
+  double tjd = 2451545.00079861 - lt;   // 0 UT as TT, corrected or light time
+
+  double RA0 = 23.86983 * DEGREE;
+  double DEC0 = 8.59590 * DEGREE;
+
+  double dRA = (23.98606 * DEGREE - RA0) / cos(DEC0);
+  double dDEC = (8.64868 * DEGREE - DEC0);
+  int n = 0;
+
+  // Planet pos;
+  radec2vector(RA0 / HOURANGLE, DEC0 / DEGREE, dist, pos1);
+
+  // Callisto's parameters from JPL Horizons
+  // https://ssd.jpl.nasa.gov/sats/elem/sep.html
+  // 1882700. 0.007 43.8  87.4  0.3 309.1 16.690440 277.921 577.264 268.7 64.8
+  sys->center = NOVAS_JUPITER;
+  novas_set_orbsys_pole(NOVAS_GCRS, 268.7 / 15.0, 64.8, sys);
+
+  orbit.jd_tdb = NOVAS_JD_J2000;
+  orbit.a = 1882700.0 * 1e3 / AU;
+  orbit.e = 0.007;
+  orbit.omega = 43.8;
+  orbit.M0 = 87.4;
+  orbit.i = 0.3;
+  orbit.Omega = 309.1;
+  orbit.n = TWOPI / 16.690440;
+  orbit.apsis_period = 277.921 * 365.25;
+  orbit.node_period = 577.264 * 365.25;
+
+  if(!is_ok("orbit_posvel_callisto", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos, vel))) return 1;
+  memcpy(pos0, pos, sizeof(pos));
+
+  for(i = 3 ;--i >= 0; ) pos[i] += pos1[i];
+  vector2radec(pos, &ra, &dec);
+
+  ra *= HOURANGLE;
+  dec *= DEGREE;
+
+  dra = (ra - RA0) * cos(DEC0);
+  ddec = (dec - DEC0);
+
+  if(!is_equal("orbit_posvel_callisto:dist", hypot(dra, ddec) / ARCSEC, hypot(dRA, dDEC) / ARCSEC, 15.0)) n++;
+  if(!is_equal("orbit_posvel_callisto:ra", dra / ARCSEC, dRA / ARCSEC, 15.0)) n++;
+  if(!is_equal("orbit_posvel_callisto:dec", ddec / ARCSEC, dDEC / ARCSEC, 15.0)) n++;
+
+  if(!is_ok("orbit_posvel_callisto:vel:null", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos1, NULL))) n++;
+  if(!is_ok("orbit_posvel_callisto:vel:null:check", check_equal_pos(pos1, pos0, 1e-8))) n++;
+
+  if(!is_ok("orbit_posvel_callisto:pos:null", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, NULL, vel1))) n++;
+  if(!is_ok("orbit_posvel_callisto:pos:null:check", check_equal_pos(vel1, vel, 1e-8))) n++;
+
+  sys->type = NOVAS_MOD;
+  if(!is_ok("orbit_posvel_callisto:mod", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos1, NULL))) n++;
+  precession(tjd, pos0, NOVAS_JD_J2000, pos);
+  j2000_to_gcrs(pos, pos);
+  if(!is_ok("orbit_posvel_callisto:mod:check", check_equal_pos(pos1, pos, 1e-8))) n++;
+
+  sys->type = NOVAS_TOD;
+  if(!is_ok("orbit_posvel_callisto:mod", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos1, NULL))) n++;
+  tod_to_j2000(tjd, NOVAS_FULL_ACCURACY, pos0, pos);
+  j2000_to_gcrs(pos, pos);
+  if(!is_ok("orbit_posvel_callisto:mod:check", check_equal_pos(pos1, pos, 1e-8))) n++;
+
+  sys->type = NOVAS_CIRS;
+  if(!is_ok("orbit_posvel_callisto:cirs", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos1, NULL))) n++;
+  cirs_to_gcrs(tjd, NOVAS_FULL_ACCURACY, pos0, pos);
+  if(!is_ok("orbit_posvel_callisto:cirs:check", check_equal_pos(pos1, pos, 1e-8))) n++;
+
+  sys->type = NOVAS_J2000;
+  if(!is_ok("orbit_posvel_callisto:j2000", novas_orbit_posvel(tjd, &orbit, NOVAS_FULL_ACCURACY, pos1, NULL))) n++;
+  j2000_to_gcrs(pos0, pos);
+  if(!is_ok("orbit_posvel_callisto:j2000:check", check_equal_pos(pos1, pos, 1e-8))) n++;
+
+  return n;
+}
+
 int main(int argc, char *argv[]) {
   int n = 0;
 
+  (void) argc;
   workPath = dirname(argv[0]);
 
   novas_debug(NOVAS_DEBUG_ON);
@@ -2066,6 +2475,23 @@ int main(int argc, char *argv[]) {
   if(test_app_hor2()) n++;
   if(test_rad_vel2()) n++;
 
+  // v1.2
+  if(test_v2z()) n++;
+  if(test_make_redshifted_object()) n++;
+  if(test_z_add()) n++;
+  if(test_z_inv()) n++;
+  if(test_redshift_vrad()) n++;
+  if(test_grav_redshift()) n++;
+
+  if(test_novas_to_naif_planet()) n++;
+  if(test_novas_to_dexxx_planet()) n++;
+  if(test_naif_to_novas_planet()) n++;
+
+  if(test_planet_for_name()) n++;
+
+  if(test_orbit_place()) n++;
+  if(test_orbit_posvel_callisto()) n++;
+
   n += test_dates();
 
   return n;
